import QtQuick 2.12
import QtQuick.Layouts 1.12
import QtQuick.Window 2.12 // For Screen
import QtWebEngine 1.8
import QtQuick.VirtualKeyboard 2.4
import QtQuick.VirtualKeyboard.Styles 2.0
import QtQuick.Controls 1.0
import Qt.labs.settings 1.0


import "."

Item {
    id: root
    
    /**
     * Default to 1366x768 with a pixel density of 2.91251 px/mm
     */
    readonly property int defaultWidth: 1366
    readonly property int defaultHeight: 768
    readonly property real defaultPixelDensity: 2.91251
    /**
     * Calculate scale factors based on ratio of current screen to default and density of pixels
     */
    readonly property real hScaleFactor: (Screen.width/defaultWidth)//*(Screen.pixelDensity/defaultPixelDensity)
    readonly property real vScaleFactor: (Screen.height/defaultHeight)//*(Screen.pixelDensity/defaultPixelDensity)
    readonly property real scaleFactor: Math.min(hScaleFactor, vScaleFactor)

    property bool keyingURL: false
    property string defaultURL: webView.fixUrl(application.homeUrl)
    property int currentContentWidth:0
    property color colorBorder: "#ba3e3e"
    property color colorBackground: "#000000"
//    property color colorUrlBar: "#ffffff"
    
    property bool fullscreen: false
    property bool tabactiveInput: false
    property bool reloadPage :false


    anchors {
        fill: parent
        leftMargin: 0
        topMargin: 0
        rightMargin: 0
        bottomMargin: 0
    }

    Connections {
        target: application

        onCommandReceived: {
            processCommand(cmd)
        }
    }

    function processCommand(cmd) {
        var parts = cmd.split(" ")
        if (parts.length < 2) {
            console.log("invalid command", cmd)
            return
        }

        switch (parts[1]) {
        case "url":
            console.log("Go to url: " + parts[2])
            webView.url = webView.fixUrl(parts[2])
            break;
        case "goback":
            controlClicked("BrowserBack")
            break;
        case "goforward":
            controlClicked("BrowserForward")
            break;
        case "gohome":
            controlClicked("BrowserHome")
            break;
        }
    }

    Rectangle {
        color: root.colorBackground
        anchors.fill: parent
    }
    
    RowLayout {
        anchors.fill: parent
        layoutDirection: Qt.RightToLeft
        spacing: 0
        
        ColumnLayout {
            Layout.fillWidth: true
            Layout.fillHeight: true
            spacing: 1 * root.vScaleFactor
/*
            Item {
                id: topbar
                Layout.fillWidth: true
                Layout.preferredHeight: root.height * 0.075 * root.vScaleFactor
                
                Image { // Unmaximize button
                    id: unmaxBtn
                    
                    anchors.top: topbar.top
                    anchors.left: topbar.left
                    anchors.bottom: topbar.bottom
                    width: 2*topbar.height
                    visible: root.fullscreen
                    
                    source: "graphics/ui3_0/unmaximize_icon.png"
                    fillMode: Image.PreserveAspectFit
                    horizontalAlignment: Image.AlignHCenter
                    verticalAlignment: Image.AlignVCenter
                    
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            root.fullscreen = false;
                        }
                    }
                }
                
                Rectangle {
                    id: urlBar
                    
                    anchors.top: topbar.top
                    anchors.bottom: topbar.bottom
                    anchors.left: root.fullscreen ? unmaxBtn.right : topbar.left
                    anchors.right: homeButton.left
                    
                    color: root.colorUrlBar
                    radius: 0
                    border.width: 1 * root.vScaleFactor
                    border.color: "transparent"

                    Rectangle {
                        id: progressBar

                        color: root.colorBorder
                        opacity: 0.6
                        radius: 0
                        width: webView.loading ? webView.loadProgress/100.0 * urlBar.width : 0
                        anchors { left: urlBar.left ; top: urlBar.top ; bottom: urlBar.bottom }
                    }

                    Flickable {
                        id: textFlickable
                        anchors { verticalCenter: parent.verticalCenter ; left: parent.left ; leftMargin: 5 * root.hScaleFactor}
                        clip: true
                        width: parent.width - clearBtn.width
                        height: urlEntry.height
                        flickableDirection: Flickable.HorizontalFlick
                        boundsBehavior: Flickable.DragOverBounds
                        interactive: contentWidth - 4 > width
                        contentWidth: urlEntry.width+2 * root.hScaleFactor
                        contentHeight: urlEntry.height

                        TextInput {
                            id: urlEntry
                            inputMethodHints: Qt.ImhUrlCharactersOnly
                            function adjustTextPosition() {
                                var offset = -1
                                var rect = urlEntry.positionToRectangle(cursorPosition)
                                if (rect.x - textFlickable.contentX + 30 > textFlickable.width) {
                                    offset = Math.max(0, rect.x - textFlickable.width)
                                    // show some trailing characters if not at the end
                                    if (cursorPosition < urlEntry.text.length - 1)
                                        offset += 30
                                }
                                if (offset >= 0)
                                    textFlickable.contentX = offset
                            }

                            width: Math.max(textFlickable.width, implicitWidth)
                            Connections {
                                target: webView
                                onUrlChanged: {
                                    urlEntry.text = webView.url
                                    urlEntry.cursorPosition = 0
                                    textFlickable.contentX = 0
                                }
                            }
                            font.pixelSize: 14*root.vScaleFactor
                            onActiveFocusChanged: {
                                if (activeFocus) {
                                    //Qt.inputMethod.show()
                                } else {
                                    //Qt.inputMethod.hide()
                                    urlEntry.cursorPosition = 0
                                    textFlickable.contentX = 0
                                }
                            }

                            onTextChanged: {
                                urlEntry.adjustTextPosition()

                            }

                            onAccepted: {
                                webView.url = webView.fixUrl(urlEntry.text)
                                webView.focus = true
                            }
                        }
                    }
                    
                    Item {
                        id: clearBtn
                        
                        property real padding: height/6
                        
                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                            right: urlBar.right
                        }
                        
                        width: clearImage.width
                        height: urlBar.height
                        
                        Image {
                            id: clearImage
                            
                            anchors {
                                top: parent.top
                                bottom: parent.bottom
                                horizontalCenter: parent.horizontalCenter
                                margins: clearBtn.padding
                            }
                            width: height*(sourceSize.width/sourceSize.height)

                            source: "graphics/ui3_0/address_clear.png"
                            fillMode: Image.PreserveAspectFit
                        }

                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                urlEntry.text = ""
                                urlEntry.focus = true
                                textFlickable.contentX = 0
                            }
                        }
                    }
                }

                Rectangle {
                    id: homeButton
                    anchors.top: topbar.top
                    anchors.bottom: topbar.bottom
                    anchors.right: topbar.right

                    width: 50 * root.hScaleFactor
                    color: "white"

                    Image {
                        id: homeImage
                        anchors {
                            fill: parent
                            margins: clearBtn.padding
                        }

                        source: "graphics/ui3_0/home_icon_grey.png"
                        fillMode: Image.PreserveAspectFit
                    }

                    MouseArea {
                        anchors.fill: parent
                        onClicked: root.controlClicked("BrowserHome")
                    }
                }
            }
 */
            GrowlMessage {
                id: loadFailedMessage
                message: '<img src="graphics/red_x.png" /> ' + qsTr("The Requested Page Could Not Be Loaded") //+ application.emptyString
                Layout.alignment: Qt.AlignTop
            }

            WebEngineView {
                id: webView

                property bool activeInput: false

                settings.pluginsEnabled: true

                //focus: true
                profile: netflixProfile
                onFullScreenRequested: {
                    console.log("fullscreen requested", request.toggleOn)
                    request.accept()
                }

                Layout.fillWidth: true
                Layout.fillHeight: true

                onNewViewRequested: {
                    if (!request.userInitiated) {
                        // Avoid opening popups
                        return
                    }else if (request.destination === WebEngineView.NewViewInTab) {
                        var tab = tabs.createEmptyTab(webView.profile)
                        tabs.visible=true
                        request.openIn(tab.item)
                    }
                    else
                    {


                    // Create new window just to grab the URL, then have it destroy itself
                    var newWindow = windowComponent.createObject(root)
                    request.openIn(newWindow.webEngineView)
                    }
                }

                url: root.defaultURL

                onUrlChanged: {
                    console.log("URL changed to " + webView.url )
                    activeInput = false
                }

                function fixUrl(url) {
                    tabs.visible=false
                    tabactiveInput=false;
                    if(url === "") return url
                    if(url[0] === "/" || url.indexOf("file:") >= 0) {
                        // Do nothing
                        return url
                    }

                    if(url.indexOf("://") < 0) {
                        if(url.indexOf(".") < 0 || url.indexOf(" ") >= 0){
                            print("url fallback ")
                            return "https://google.com/search?q=" + url
                        }
                        else
                            return urlRewrite("http://" + url)
                    }
                    else {
                        return urlRewrite(url)
                    }
                }

                function urlRewrite(url) {
                    return url
                }

                onLoadingChanged: {
                    if (loadRequest.status === WebEngineView.LoadFailedStatus) {
                        console.log("Failed to load webpage", webView.url, webView.progress)
                        if(root.reloadPage===false)
                        {
                             webView.reload()
                            console.log("reload the page **********************")
                            root.reloadPage=true
                        }
                        loadFailedMessage.show(3)
                    } else if (loadRequest.status === WebEngineView.LoadStartedStatus) {
                        // Hide keypad when going to a new page
                        //Qt.inputMethod.hide()
                        //keypadManager.hideKeypad("inputKeypad")
                    } else if (loadRequest.status === WebEngineView.LoadSucceededStatus) {

                        // Check to see if the Netflix icon was selected
                        if (webView.title.indexOf("Netflix") >= 0) {
                            webView.profile = netflixProfile
                            console.log("Setting profile to netflix")
                        } else {
                             webView.profile = netflixProfile
                           // webView.profile = privateBrowsing
                            console.log("Setting profile to netflix")
                        }

                        webView.runJavaScript("
if (window.onbeforeunload) {
// Prevent creating dialogs as user leaves page
var original_func = window.onbeforeunload;
window.onbeforeunload = function(e) {
  original_func(e);
  return null; // Do not show a dialog
};
}
// Suppress alert messages. We didn't build the module
// required to display them (QtQuick.Dialogs) and they
// misbehave when the module is installed.
window.alert = function(msg) { console.log('Alert:', msg); }
window.prompt = function(msg) { return null; }

var TrueFitness_log_message = function(msg) {
  // Some sites (e.g. twitter) override console.log to be an empty function
  // Therefore use console.error, which is at least LESS likely to be overridden
  console.error(msg);
}

var isInputField = function(element) {
  if (!element) {
    return false;
  }

  if (element.tagName !== 'INPUT' && element.tagName !== 'TEXTAREA') {
    return false;
  }
  if (element.tagName === 'TEXTAREA') {
    return true;
  }
  if (element.tagName === 'INPUT') {
    switch (element.type) {
      case 'text':
      case 'password':
      case 'search':
      case 'email':
      case 'url':
      case 'tel':
      case 'number':
        return true;
      default:
        return false;
    }
  }

  return false;
}
/*************************espn*******************/
var isEspnInput=function(element){
    TrueFitness_log_message(element.tagName);
    if(element.tagName==='A'){
        return true;
    }
    return false;
}


var isFacebookInput = function(element) {
  // Special case for Facebook - they don't use normal input fields
  // Warning: Brittle
  if (element.tagName === 'DIV') {
    if (element.getAttribute('data-offset-key') !== null) {
      return true;
    }
    if (element.getAttribute('aria-label') === 'Search Facebook') {
      return true;
    }
    return false;
  }

  return false;
}

/* Note: We set the third parameter (useCapture) to true as this ensures that the window-level
 * events run first, rather than having to bubble up from the input element. I.e. it is
 * more efficient.
 */
window.addEventListener('focus',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isInputField(e.target)) {
    TrueFitness_log_message('@InputFocus');
  }
}, true);

window.addEventListener('blur',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isInputField(e.target)) {
    TrueFitness_log_message('@InputBlur');
  }
}, true);

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isInputField(e.target)) {
    if (e.target === document.activeElement) {
      // Only fire if this element is clicked after already gaining focus
      TrueFitness_log_message('@InputClick');
    }
  }
}, true);
/*********************plus.espn.com****************/
if (window.location.href.indexOf('plus.espn.com') >= 0) {

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;

  }
if(isEspnInput(e.target)){
TrueFitness_log_message('@InputFocus');
}else {
    if (!isInputField(e.target)) {
      TrueFitness_log_message('@InputBlur');
    }
  }

}, true);

}

/******************espn*********************/

if (window.location.href.indexOf('espn.com') >= 0) {

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;

  }
if(isEspnInput(e.target)){
TrueFitness_log_message('@InputFocus');
}else {
    if (!isInputField(e.target)) {
      TrueFitness_log_message('@InputBlur');
    }
  }

}, true);

}


// -- Special cases for Facebook --
// Facebook does not use normal input fields, but DIV-based react.js fields
if (window.location.href.indexOf('facebook.com') >= 0) {

window.addEventListener('click',
function(e) {
  if (!e || !e.target) {
    return;
  }

  if (isFacebookInput(e.target)) {
    TrueFitness_log_message('@InputFocus');
  } else {
    // Anything that is not a FB input field, or traditional input
    // field, is treated as a blur event.
    if (!isInputField(e.target)) {
      TrueFitness_log_message('@InputBlur');
    }
  }
}, true);

}
// -- End Facebook special case --

// Check if an input field is already selected, and show keypad if so
if (false) { // Disable for now
  if (document && document.activeElement) {
    if (isInputField(document.activeElement)) {
      TrueFitness_log_message('@InputFocus');
    }
  }
}
");
                    }
                }

                onJavaScriptConsoleMessage: {
                    console.log(message);

                    if (message[0] === "@") {
                        if (message === "@InputFocus" || message === "@InputClick") {
                            activeInput = true
                        } else if (message === "@InputBlur") {
                            activeInput = false
                        }
                    }
                }

                transformOrigin: Item.TopLeft
                smooth: true
            } // end WebEngineView
        } // end ColumnLayout
    } // end RowLayout


    TabView {
            id: tabs
            visible: false
            frameVisible: false
            tabsVisible: false
            function createEmptyTab(profile) {
                if(tabs.currentIndex==1)
                {
                    tabs.removeTab(tabs.currentIndex);
                    tabactiveInput=false;

                }
                tabactiveInput=true

                var tab = addTab("", tabComponent);
                // We must do this first to make sure that tab.active gets set so that tab.item gets instantiated immediately.
                tab.active = true;
                tab.item.profile = netflixProfile;
                return tab;
            }

            anchors.fill: parent


            Component {
                id: tabComponent
                WebEngineView {
                    id: webEngineView

                    focus: true
                     profile: netflixProfile
                     settings.autoLoadImages: true
                    settings.javascriptEnabled: true
                    settings.errorPageEnabled: true
                    settings.pluginsEnabled: true
                    settings.fullScreenSupportEnabled: true
                    settings.autoLoadIconsForPage: true
                    settings.touchIconsEnabled: true
                    settings.webRTCPublicInterfacesOnly: true

                    onWindowCloseRequested: {
                        console.log("close the window",tabs.currentIndex);

                         tabs.removeTab(tabs.currentIndex);
                        tabs.visible=false;
                        tabactiveInput=false;
                        }
                }
            }


        }

   

    property Component windowComponent: Item {
        visible: false

        property WebEngineView webEngineView: webEngineView_
        WebEngineView {
            id: webEngineView_
            profile: netflixProfile
            anchors.fill: parent
            onUrlChanged: {
                if (!url.toString()) {
                    return
                }
                console.log("Opening", url, "in current window rather than new window")
                webView.url = url
                parent.destroy()
            }
        }
    }

    WebEngineProfile {
        id: privateBrowsing
        offTheRecord: false
        //persistentCookiesPolicy: WebEngineProfile.NoPersistentCookies
    }

    WebEngineProfile {
        id: netflixProfile
        // Netflix requires at least offTheRecord to be false for its encryption to work
        // The other properties are an attempt to tighten the number of other items that get stored
        offTheRecord: false
        persistentCookiesPolicy: WebEngineProfile.NoPersistentCookies
        persistentStoragePath: application.persistentStoragePath
        httpCacheType: WebEngineProfile.MemoryHttpCache
        cachePath: ""
    }

    ListModel {
        id: controlsList
        ListElement {
            name: "BrowserBack"
            label: ""
            iconPath: "graphics/ui3_0/back_triangle_white.png"
        }
        ListElement {
            name: "BrowserForward"
            label: ""
            iconPath: "graphics/ui3_0/forwrd_triangle_white.png"
        }
        ListElement {
            name: "BrowserHome"
            label: ""
            iconPath: "graphics/ui3_0/Home_icon.png"
        }
        ListElement {
            name: "BrowserFullScreen"
            label: ""
            iconPath: "graphics/ui3_0/fullscreen_icon.png"
        }
    }

    function controlClicked(name)
    {
        if(name === "BrowserBack") {
            if(webView.canGoBack) {
                webView.goBack()
            }
        } else if(name === "BrowserForward") {
            if(webView.canGoForward) {
                webView.goForward()
            }
        } else if(name === "BrowserHome") {
            webView.url = root.defaultURL
        } else if(name === "BrowserFullScreen") {
            root.fullscreen = !root.fullscreen;
        }
    }

    TrueKeyboard {
        id: keyboard
        width: root.width * 0.75
        anchors.horizontalCenter: root.horizontalCenter
        active: webView.activeInput | tabactiveInput //|| urlEntry.activeFocus
        z: 999999
    }

    Connections {
        target: InputContext.inputEngine
        onPreviousKeyChanged: {
            if(webView.activeInput) {
                if(InputContext.shift) {
                    InputContext.shift = false
                }
            }
        }
    }

}
